# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

import shutil
import tempfile
from os import makedirs
from os.path import dirname, exists, join

import pytest
from flask import Blueprint, Flask
from flask.cli import ScriptInfo

from invenio_assets import InvenioAssets


@pytest.yield_fixture()
def instance_path():
    """Temporary instance path."""
    path = tempfile.mkdtemp()
    yield path
    shutil.rmtree(path)


@pytest.yield_fixture()
def static_dir():
    """Static file directory."""
    static_dir = join(dirname(__file__), 'static')
    if not exists(static_dir):
        makedirs(static_dir)
    yield static_dir
    shutil.rmtree(static_dir)


@pytest.yield_fixture()
def testcss(static_dir):
    """Test CSS file."""
    filepath = join(static_dir, 'test.css')
    with open(filepath, 'w') as fp:
        fp.write('* {color: white;}')
    yield filepath


@pytest.yield_fixture()
def app(instance_path):
    """Flask application fixture."""
    app = Flask(
        __name__,
        instance_path=instance_path,
        static_folder=join(instance_path, 'static')
    )
    app.config.update({
        'COLLECT_STORAGE': 'flask_collect.storage.file',
    })

    yield app


@pytest.yield_fixture()
def script_info(app):
    """Get ScriptInfo object for testing CLI."""
    InvenioAssets(app)
    yield ScriptInfo(create_app=lambda info: app)


@pytest.yield_fixture()
def script_info_assets(app, static_dir, testcss):
    """Get ScriptInfo object for testing CLI."""
    InvenioAssets(app)

    blueprint = Blueprint(
        __name__, 'test_bp', static_folder=static_dir
    )

    class Ext(object):
        def __init__(self, app):
            assets = app.extensions['invenio-assets']
            app.register_blueprint(blueprint)

    Ext(app)

    yield ScriptInfo(create_app=lambda info: app)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test Webpack module."""

import pytest
from flask_webpackext import WebpackBundle
from pywebpack import UnsupportedExtensionError

from invenio_assets.webpack import UniqueJinjaManifestEntry, \
    UniqueJinjaManifestLoader, WebpackThemeBundle


def test_webpack_theme_bundle_outside_app():
    """Access a bundle property outside app context."""
    # Test that bundles are created for each theme.
    bundle = WebpackThemeBundle(
        'tests',
        'assets',
        default='semantic-ui',
        themes={
            'semantic-ui': dict(
                entry={
                    'theme': './theme-semantic-ui.css',
                }
            ),
        }
    )
    pytest.raises(AttributeError, getattr, bundle, 'path')


def test_webpack_theme_bundle(app):
    """Test WebpackThemeBundle."""
    themes = {
        'bootstrap3': dict(
            entry={
                'theme': './theme-bootstrap3.css',
            }
        ),
        'semantic-ui': dict(
            entry={
                'theme': './theme-semantic-ui.css',
            }
        ),
    }

    # Test that bundles are created for each theme.
    bundle = WebpackThemeBundle(
        'tests', 'assets', default='semantic-ui', themes=themes)
    assert isinstance(bundle.themes['bootstrap3'], WebpackBundle)
    assert isinstance(bundle.themes['semantic-ui'], WebpackBundle)

    # Test that default theme is used.
    with app.app_context():
        assert bundle._active_theme_bundle == bundle.themes['semantic-ui']
        assert bundle.path == bundle.themes['semantic-ui'].path
        assert bundle.entry == bundle.themes['semantic-ui'].entry
        assert bundle.dependencies == bundle.themes['semantic-ui'].dependencies

    # Test that APP_THEME overrides default theme.
    app.config['APP_THEME'] = ['bootstrap3']
    with app.app_context():
        assert bundle._active_theme_bundle == bundle.themes['bootstrap3']
        assert bundle.path == bundle.themes['bootstrap3'].path
        assert bundle.entry == bundle.themes['bootstrap3'].entry
        assert bundle.dependencies == bundle.themes['bootstrap3'].dependencies

    # Test that an invalid APP_THEME
    app.config['APP_THEME'] = ['invalid']
    with app.app_context():
        assert bundle._active_theme_bundle is None


def test_unique_jinja_manifest_entry(app):
    """Test UniqueJinjaManifestEntry."""
    # b.js is only output once, despite being twice in manifest
    m = UniqueJinjaManifestEntry('manifestname', ['/a.js', '/b.js', '/b.js'])
    with app.test_request_context():
        assert m.__html__() == \
            '<script src="/a.js"></script>\n' \
            '<script src="/b.js"></script>'

    # b.js is only output once, despite being twice in manifest
    app.debug = True
    with app.test_request_context():
        assert m.__html__() == \
            '<!-- manifestname -->\n' \
            '<script src="/a.js"></script>\n' \
            '<script src="/b.js"></script>'

    # Unsupported file extension
    m = UniqueJinjaManifestEntry('script', ['/a.less'])
    with app.test_request_context():
        pytest.raises(UnsupportedExtensionError, m.__html__)


def test_unique_jinja_manifest_loader(app):
    """Test UniqueJinjaManifestLoader."""
    loader = UniqueJinjaManifestLoader()
    assert loader.entry_cls == UniqueJinjaManifestEntry



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Media asset management for Invenio."""

from functools import partial

import pkg_resources
from flask_collect import Collect
from flask_webpackext import FlaskWebpackExt

from .collect import collect_staticroot_removal
from .webpack import UniqueJinjaManifestLoader

__all__ = ('InvenioAssets', )


class InvenioAssets(object):
    """Invenio asset extension."""

    def __init__(self, app=None, **kwargs):
        r"""Extension initialization.

        :param app: An instance of :class:`~flask.Flask`.
        :param \**kwargs: Keyword arguments are passed to ``init_app`` method.
        """
        if app:
            self.init_app(app, **kwargs)

    def init_app(self, app, **kwargs):
        """Initialize application object.

        :param app: An instance of :class:`~flask.Flask`.

        .. versionchanged:: 1.0.0b2
           The *entrypoint* has been renamed to *entry_point_group*.
        """
        self.init_config(app)

        self.collect = Collect(app)
        self.webpack = FlaskWebpackExt(app)

        app.extensions['invenio-assets'] = self

    def init_config(self, app):
        """Initialize configuration.

        :param app: An instance of :class:`~flask.Flask`.
        """
        # Flask-Collect config
        app.config.setdefault('COLLECT_STATIC_ROOT', app.static_folder)
        app.config.setdefault('COLLECT_STORAGE', 'flask_collect.storage.link')
        app.config.setdefault(
            'COLLECT_FILTER', partial(collect_staticroot_removal, app))

        # Flask-WebpackExt config
        app.config.setdefault(
            'WEBPACKEXT_PROJECT', 'invenio_assets.webpack:project')
        app.config.setdefault(
            'WEBPACKEXT_MANIFEST_LOADER', UniqueJinjaManifestLoader)
        if app.debug:  # for development use 2-level deep symlinking
            from pywebpack.storage import LinkStorage
            app.config.setdefault(
                'WEBPACKEXT_STORAGE_CLS', partial(LinkStorage, depth=2))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Media assets management for Invenio.

Invenio-Assets helps you with integration of webassets, installation of NPM
packages, and process of collecting static files.

Initialization
--------------
First create a Flask application:

    >>> from flask import Flask
    >>> app = Flask('myapp')

Next, initialize your extension:

    >>> from invenio_assets import InvenioAssets
    >>> assets = InvenioAssets(app)

During initialization two Flask extensions
:class:`flask_webpackext.ext.FlaskWebpackExt` and
:class:`flask_collect.Collect` are instantiated and configured.

Bundles specified in the entry point groups called ``invenio_assets.webpack``
are automatically registered by Invenio-Assets.

.. _use-flask-webpackext:

Using Flask-WebpackExt
----------------------

Bundles
~~~~~~~
The `Flask-WebpackExt <https://flask-webpackext.readthedocs.io/en/latest/>`_
package provides a class
:class:`flask_webpackext.bundle.WebpackBundle` for declaring the needed
assets and NPM dependencies of each one of your modules.

.. code-block:: python

    from flask_webpackext import WebpackBundle
    bundle1 = WebpackBundle(
        __name__,
        './modules/module1/static',
        entry={
            'module1-app': './js/module1-app.js',
        },
        dependencies={
            'jquery': '^3.2.1'
        }
    )

The NPM dependencies defined in the bundles will be used to generate the
``package.json`` file.

Entry points loading
++++++++++++++++++++

Invenio-Assets will automatically load bundles defined by the entry point
group ``invenio_assets.webpack``. Example:

.. code-block:: python

   # setup.py
   setup(
       # ...
       entry_points={
           'invenio_assets.webpack': [
               'mybundle = mypackage.bundles:mybundle',
           ],
           # ...
        }
        # ...
    )

Command-line interface
~~~~~~~~~~~~~~~~~~~~~~

We can now build the assets:

.. code-block:: console

    $ flask webpack buildall

The command will copy all files from the src folder to the application
instance folder designated for the Webpack project, download the npm packages
and run Webpack to build our assets.

Alternatively, we can execute each build step separately with the following
flask webpack commands:

    * ``create`` - Copy all sources to the working directory.
    * ``install`` - Run npm install command and download all dependencies.
    * ``build`` - Run npm run build.

Additionally if we have some static files we should collect them:

.. code-block:: console

    $ flask collect -v
"""

from .ext import InvenioAssets
from .proxies import current_assets
from .version import __version__

__all__ = (
    '__version__',
    'InvenioAssets',
    'current_assets',
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Test Invenio Assets module."""

from mock import patch
from pywebpack.storage import FileStorage, LinkStorage

from invenio_assets import InvenioAssets


def test_version():
    """Test version import."""
    from invenio_assets import __version__
    assert __version__


def test_init(app):
    """Test module initialization."""
    InvenioAssets(app)
    assets = app.extensions['invenio-assets']
    assert assets.collect
    assert assets.webpack
    assert app.config['COLLECT_STATIC_ROOT'] == app.static_folder
    assert 'WEBPACKEXT_PROJECT' in app.config


def test_init_post(app):
    """Test module initialization using init_app."""
    assets = InvenioAssets()
    assert 'COLLECT_STATIC_ROOT' not in app.config
    assets.init_app(app)
    assert assets.collect
    assert assets.webpack
    assert 'COLLECT_STATIC_ROOT' in app.config
    assert 'WEBPACKEXT_PROJECT' in app.config
    assert app.config['WEBPACKEXT_STORAGE_CLS'] == FileStorage


def test_init_cli(app):
    """Test cli registration."""
    from flask.cli import cli
    cli._load_plugin_commands()
    assert 'collect' in cli.commands
    assert 'webpack' in cli.commands


def test_init_debug(app):
    """Test module initialization with debug enabled."""
    app.debug = True
    InvenioAssets(app)
    # Storage class changed to LinkStorage when in debug mode.
    assert app.config['WEBPACKEXT_STORAGE_CLS'] != FileStorage



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2020 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test Invenio Assets module."""

from os.path import exists, isfile, join
from time import sleep

from click.testing import CliRunner

from invenio_assets.cli import collect


def test_collect(app, script_info_assets, testcss):
    """Test assets command in assets CLI."""
    css_path = join(app.extensions['collect'].static_root, 'test.css')

    # Run collect
    runner = CliRunner()
    assert not exists(css_path)
    result = runner.invoke(collect, ['-v'], obj=script_info_assets)
    assert result.exit_code == 0
    assert "Copied: [conftest] '{0}'".format(css_path) in result.output

    # Run collect again - no file copied
    result = runner.invoke(collect, ['-v'], obj=script_info_assets)
    assert result.exit_code == 0
    assert 'Copied' not in result.output

    # Modify file (and ensure timestamp is different)
    sleep(1)
    with open(testcss, 'w') as fp:
        fp.write('* {color: black;}')

    # Run collect again - file will be copied
    result = runner.invoke(collect, ['-v'], obj=script_info_assets)
    assert result.exit_code == 0
    assert 'Copied' in result.output
